// Template file v5.202 (02/20/02)
////////////////////////////////////////////////////////////////////////
// File: animate.wdl
//		WDL prefabs for model animation
////////////////////////////////////////////////////////////////////////
// Use:
//		Include AFTER "movment.wdl"
//
//]- Created: 06/14/01 DCP
//]-
//]- Mod Date: 07/10/01 DCP
//]-		actor_anim(): Added check for *ADVANCED ANIMATION*
//]-		actor_adv_anim(): Created. Uses SKILLS 1,2,3 & 4 for animation
//]-
//]-	Mod Date:  08/16/01  DCP
//]-		move_shadow(): Fixed drop shadow alignment on slopes.
//]-
//]- Mod Date: 12/18/01
//]-		Added WED 'uses' comments to all WED editable fields



//@ Animate Vars
var anim_attack_ticks = 16;// time for one attack animation cycle

var anim_stand_ticks = 16;	// time for one standing anim cycle
var anim_jump_ticks = 6; 	// time for one jump animation cycle
var anim_duck_ticks = 8; 	// time for one duck animation cycle

var anim_walk_dist = 1; 	// dist per model width per walk cycle
var anim_run_dist = 1.5;	// dist per model width per run cycle
var anim_crawl_dist = 0.8; // dist per model width per crawl cycle
var anim_wade_dist = 0.8;	// dist per model width per crawl cycle
var anim_swim_dist = 1; 	// dist per model width per swim cycle

var walk_or_run = 12; 	// max quants per tick to switch from walk to run animation


// strings for defining the animation frames
STRING anim_stand_str,"stand";
STRING anim_walk_str,"walk";
STRING anim_run_str,"run";
STRING anim_duck_str,"duck";
STRING anim_swim_str,"swim";
STRING anim_dive_str,"dive";
STRING anim_jump_str,"jump";
STRING anim_crawl_str,"crawl";
STRING anim_wade_str,"walk";

// string synonyms for defining the animation frames (and their default values)
STRING anim_default_death_str,"death";
STRING anim_default_attack_str,"attack";
//SYNONYM anim_attack_str { TYPE STRING; DEFAULT anim_default_attack_str;}
string* anim_attack_str = anim_default_attack_str;
//SYNONYM anim_death_str { TYPE STRING; DEFAULT anim_default_death_str;}
string* anim_death_str = anim_default_death_str;

//SYNONYM anim_str { TYPE STRING; }
string* anim_str;


// Define Masks
DEFINE MASK_ANIM_ATTACK_TICKS,2093056;	// upper 9 (less topbit)
DEFINE MASK_ANIM_JUMP_TICKS,4032; 		// mid 6
DEFINE MASK_ANIM_DUCK_TICKS,63; 			// lower 6

DEFINE MASK_ANIM_RUN_DIST,	2064384;	// upper 6 (less topbit)
DEFINE MASK_ANIM_WALK_DIST, 31744;	// next 5
DEFINE MASK_ANIM_CRAWL_DIST, 992;	// next 5
DEFINE MASK_ANIM_SWIM_DIST, 31;		// bottom 5


//@ Function prototypes

//function anim_airborne();   // animate 'airborne' entity
function anim_init();		// initialize the actor animation style
//function actor_anim();		// handle actor animation
//function	actor_anim_transition(str_anim_target);	// handle transition animation between states
//function actor_anim_old_style_anim();	// old animation style (for compatability)

//#ACTION drop_shadow;			// create a shadow sprite under entity (use move_shadow() to move)
function move_shadow();		// move shadow sprite under entity
function attach_entity();	// attach an entity with same origin/frames

//@ Function code

//////////////////////////////////////////////////////////////////////////
// =======================================================================
// ANIMATION '2' functions (animation with blending, animation that makes sense)


// Desc: animate a blended transition between two states
//
//  'my' must be set to an actor (i.e. my = player)
//  my._ANIMDIST is set to 0 at the end of the transition, it is equal
//to -1 the rest of the time.
//
// str_anim_source is a string containing the source animation name
// str_anim_target is a string containing the target animation name
// trans_ticks is the number of ticks this function will take to complete
//
// e.g. actor_anim_transition2("walk","stand",16);
//		- blend from walk to stand in 16 ticks (one second)

// pointers to animation strings
string*	pStr_old_anim;	// older animation string
string*	pStr_cur_anim;	// current animation string

// stand, walk, run, duck, crawl, jump, swim, attack, and death
DEFINE	_ANIM_MODE_QUIT,-1;	// quit animation
DEFINE	_ANIM_MODE_NONE,0;	// no animation
DEFINE	_ANIM_MODE_STAND,1;	// standing/idle animation (tick)
DEFINE	_ANIM_MODE_WALK,2;	// walking animation (dist)
DEFINE	_ANIM_MODE_RUN,3;		// running animation (dist)
DEFINE	_ANIM_MODE_WADE,4;	// wading animation
DEFINE	_ANIM_MODE_DUCK,5;	// ducking animation	(?)
DEFINE	_ANIM_MODE_CRAWL,6;	// crawling animation (?)
DEFINE	_ANIM_MODE_JUMP,7;	// jumping animation (tick)
DEFINE	_ANIM_MODE_SWIM,8;	// swimming animation (?)
DEFINE	_ANIM_MODE_ATTACK,9;	// attack animation
DEFINE	_ANIM_MODE_DEATH,10;  // dying animation

// if the movement distance is less than this value, the actor/player
//is concidered to be 'standing still' as far as the advanced animation
//functions are concerned (so it will choose to play the idle animations)
var anim_stillness_threshold = 0.025;

// Desc: get the animation string for the current animatino mode
//
// calls:
// called by: actor_adv_anim2
// uses: MY._ADVANIM_TICK, MY._ADVANIM_DIST (read only)
//
//
// set pStr_cur_anim to the right string
// return the cycle value if quant based or negitive cycle if tick based
// Desc: evaluate the entity and select the correct animation mode
//
// calls:
// called by: actor_adv_anim2
//
//	takes: cur_anim_mode - current animation mode
//	uses:^my._FIREMODE  - see if actor is shooting
//		  ^my._MOVEMODE  - find out what movement state actor is in
//		  ^my._WALKDIST  - how far (quants) the actor moved last frame
//		  ^MY._ADVANIM_DIST - to calculate walk/run distance
//			temp
//			temp2
//
// returns the animation mode
// sets temp to 1 if this animation is 'once only' (e.g. jump, fire)
//
// Mod Date: 06/07/02 DCP
//		Time corrected MY._WALKDIST in calculations

// Desc: new animation method
//		- uses transitions
//		- loops every frame
//
//   'my' must be set before calling
//		my._WALKDIST must be set to the distance that the actor has covered (in quants)
//		my._ANIMDIST is modified in this function
//
//    global pStr_cur_anim is used
//
// Mod Date: 05/01/02 DCP
//		Once Off animations now use ent_frame instead of ent_cycle
//		Added a check for "no animation"
//
// Mod Date: 05/08/02 DCP
//		Fixed problem with multiple actions calling this function effecting the 'blend cycle'
//  We now make an aditional call to '_actor_adv_anim2_get_string'
//
// Mod Date: 06/07/02 DCP
//		Added "proc_late()" call to make sure this function is called only after
// MY._WALKDIST is set for the frame.
//
// Mod Date: 07/02/02 DCP
//		Added local var 'anim_percent' to store percent of animation cycle
//  Assign this value to my._ANIMDIST before calling actor_anim_transition2
//	 (bug fix: since actor_anim_transition2 expects _ANIMDIST to be a percent
//	 not a raw number).









// =======================================================================
//////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////
// Desc: set up for animation
// 		scale entity by actor_scale
//			if using the "old style animation"
//	      	- split the integer and fractional parts of the animation
//				  frame numbers, and store distance factors
//
//]- Mod Date: 8/31/00 DCP
//]-		Scale the player by vec_scale
function anim_init()
{
	vec_scale(MY.SCALE_X,actor_scale);

	if(int(MY._WALKFRAMES) < 0) { return; }	// use adv animation

	// init 'old style' animation
	temp = frc(MY._WALKFRAMES) * 1000;
	if(temp != 0)
	{
		// old style animation
		MY._WALKFRAMES = int(MY._WALKFRAMES);
		if(MY._WALKFRAMES == 0) { MY._WALKFRAMES = 13; }
		MY._WALKDIST = MY._WALKFRAMES / temp;

		temp = frc(MY._RUNFRAMES) * 1000;
		MY._RUNFRAMES = int(MY._RUNFRAMES);
		if(MY._RUNFRAMES == 0) { MY._RUNFRAMES = 5; }
		MY._RUNDIST = MY._RUNFRAMES / temp;
	}
}



/////////////////////////////////////////////////////////////////////////
// Desc: Handle 'old' A4 style of animation
//			Called from "actor_anim()"
//
//]-	Created: 06/11/01 DCP
//]-
/*
function drop_my_shadow()
{
	if (MY.SKILL53==NULL)
	{
	//MY.SKILL53=
	ent_create(SHADOWSPRITE,nullvector,NULL);
	You.transparent = ON;
	You.alpha = 60;
	You.passable = ON;
	You.oriented = ON;
	You.unlit = ON;		 //(plus UNLIT flag in version 4.20)
	//MY.scale_x = (my.MAX_X - my.MIN_X)/(MY.MAX_X - MY.MIN_X);
	//MY.scale_y = MY.scale_x * 0.8;
	//MY.scale_z = 1.0;
	wait(1);
	}
	
				if(result > 0)
			{
				// place shadow 2 quants above the floor
				MY.z = YOU.z - RESULT + 2;  + 2 - RESULT;
				MY.x = YOU.x;
				MY.y = YOU.y;
//--	 		 	MY.pan = YOU.pan;

				// adapt shadow orientation to floor slope
				if ((NORMAL.x != 0) || (NORMAL.y != 0))
				{ // we're on a slope
					// conform shadow to slope
					MY.PAN = 0;
					MY.tilt = - asin(NORMAL.x);
					MY.roll = - asin(NORMAL.y);
					temp.pan = YOU.PAN;
					temp.tilt = 90;
					temp.roll = 0;
					rotate(my,temp,nullvector);
 				}
				else
				{
					MY.pan = YOU.pan;
					MY.tilt = 90; // set it flat onto the floor
					MY.roll = 0;
				}
			}
			else
			{
				MY.INVISIBLE = ON;
			}

	
	
}
*/
/////////////////////////////////////////////////////////////////////////
// Desc: create a shadow below the entity
ACTION drop_shadow
{
IFDEF CAPS_FLARE;
	if(VIDEO_DEPTH >= 16)
	{
		create(SHADOWSPRITE,MY.POS,move_shadow);
	}
	else
	{
		create(SHADOWFLAT,MY.POS,move_shadow);
	}
IFELSE;
	create(SHADOWFLAT,MY.POS,move_shadow);
ENDIF;
}



/////////////////////////////////////////////////////////////////////////
//	Desc: function used to move shadow
//
//]- Mod Date: 05/29/00
//]- 			Added check for swimming or wading (no shadow)
//]- Mod Date: 7/30/00 JCL
//]-				Made shadow darker + adapted to floor slope
//]- Mod Date: 11/9/00 DCP
//]-				Replaced sonar with trace()
//]-	Mod Date: 08/16/01 DCP
//]-		Fix conform shadow to slope.
function move_shadow()
{
	MY.transparent = ON;
//	MY.flare = ON;
	MY.alpha = 60;
	MY.passable = ON;
	MY.oriented = ON;
//	MY.ambient = -100; // shadow should be totally black
	MY.unlit = ON;		 //(plus UNLIT flag in version 4.20)

	// scale the shadow so that it matches its master's (YOU) size
	MY.scale_x = (YOU.MAX_X - YOU.MIN_X)/(MY.MAX_X - MY.MIN_X);
	MY.scale_y = MY.scale_x * 0.8;
	MY.scale_z = 1.0;

	while(YOU != NULL)
	{
		if (YOU._HEALTH<=0) {break;}
		
		if ((YOU.invisible == ON)
		|| (YOU._MOVEMODE == _MODE_SWIMMING)
		|| (YOU._MOVEMODE == _MODE_WADING))
		{
			MY.invisible = ON;
		}
		else
		{
			MY.invisible = OFF;
  			temp_ent = YOU;


			//-old-sonar temp_ent,500; // get height above the floor
			trace_mode = IGNORE_PASSENTS
			  	+ IGNORE_ME
			  	+ IGNORE_YOU
			  	+ IGNORE_MODELS
			  	+ IGNORE_SPRITES
			  	+ ignore_push;
			vec_set(vecFrom,YOU.X);
			vec_set(vecTo,vecFrom);
		
		//	vec_set(vecFrom,YOU.X);
		//	vecFrom+=YOU.MIN_Z;
			
			
		//	vec_set(vecTo,vecFrom);
		
			vecTo.Z -= 124;
			result = trace(vecFrom,vecTo);

			YOU = temp_ent; // YOU (the entity itself) is changed by SONAR
	
//--	 	

			if(result > 0)
			{
				// place shadow 2 quants above the floor
				MY.z = YOU.z - RESULT + 2; 
				MY.x = YOU.x;
				MY.y = YOU.y;
//--	 		 	MY.pan = YOU.pan;

				// adapt shadow orientation to floor slope
				if ((NORMAL.x != 0) || (NORMAL.y != 0))
				{ // we're on a slope
					// conform shadow to slope
					MY.PAN = 0;
					MY.tilt = - asin(NORMAL.x);
					MY.roll = - asin(NORMAL.y);
					temp.pan = YOU.PAN;
					temp.tilt = 90;
					temp.roll = 0;
					rotate(my,temp,nullvector);
 				}
				else
				{
					MY.pan = YOU.pan;
					MY.tilt = 90; // set it flat onto the floor
					MY.roll = 0;
				}
			}
			else
			{
				MY.INVISIBLE = ON;
			}
			
		}
		wait(4);
	} // end while(YOU != NULL)
	remove(ME);
}


/////////////////////////////////////////////////////////////////////////
// Desc: attaches an entity that has the same origin and the same frame cycles
//
function attach_entity()
{
   my.passable = on;
   while(you)	// prevent empty synoym error if parent entity was removed
   {
   	if(your.shadow == on) { my.shadow = on; }
   	if(you == player && person_3rd == 0)
		{
      	my.invisible = on;
   	}
		else
		{
      	my.invisible = off;
   	}
   	vec_set(my.x,you.x);
   	vec_set(my.pan,you.pan);
   	my.frame = you.frame;
   	my.next_frame = you.next_frame;

   	wait(1);
   }
	remove(my);
}


///////////////////////////////////////////////////////////////////////
// Desc: Shakes the player, used for hits and death
function player_shake()
{
	if(random(1) > 0.5)
	{
		MY.ROLL += 8;
		MY.TILT += 8;
		waitt(2);
		MY.TILT -= 5;
		waitt(2);
		MY.ROLL -= 8;
		MY.TILT -= 3;
	}
	else
	{
		MY.ROLL -= 8;
		MY.TILT += 8;
		waitt(2);
		MY.TILT -= 5;
		waitt(2);
		MY.ROLL += 8;
		MY.TILT -= 3;
	}
}